package com.zhili.zkservicecenter.component;


import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.zhili.zkservicecenter.configuration.ServerConfig;
import com.zhili.zkservicecenter.dto.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.curator.framework.CuratorFramework;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.script.RedisScript;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

@Component
@Slf4j
public class KafkaMessageHandler {

    @Autowired
    ServerConfig serverConfig;

    @Autowired
    RedisTemplate redisTemplate;

    @Autowired
    RedisHelper redisHelper;

    @Autowired
    ServiceCenter serviceCenter;

    @Autowired
    KafkaTemplate kafkaTemplate;

    @Resource(name = "subscribeExchangeScript")
    RedisScript<List> subscribeScript;

    public KafkaMessageHandler(CuratorFramework client) {
    }

    @KafkaListener(topics = {"#{serverConfig.getServiceName()}_ExchangeEvent"}, groupId = "#{serverConfig.getServiceName()}")
    public void processExchangeEvent(ConsumerRecord<String, String> record, Acknowledgment ack) {
        String value = record.value();
        ExchangeEvent exchangeEvent = JSON.parseObject(value, ExchangeEvent.class);
        log.info("recv exchangeEvent:" + exchangeEvent);
        String exchangeNo = exchangeEvent.getExchangeNo();
        String stationCode = exchangeEvent.getStationCode();
        //注册站的当前换电号
        //{"StationCurrentExchange_st123":"ex_abcdef"}
        redisTemplate.opsForValue().set("StationCurrentExchange:" + stationCode, exchangeEvent, 12, TimeUnit.HOURS);
        //换电详情缓存成list
        redisTemplate.opsForList().rightPush("ExchangeDetail:" + exchangeNo, exchangeEvent);
        redisTemplate.expire("ExchangeDetail:" + exchangeNo, 12, TimeUnit.HOURS);
        //循环正在监听当前换电号的wx服务器连接
        Set<String> serverSessions = redisHelper.scan("SessionsSubscribeExchange:" + exchangeNo + ":");
        log.info("serverSessions：" + serverSessions);
        for (String key : serverSessions) {
            String[] split = key.split(":");
            DriverConnInfo driverConnInfo = new DriverConnInfo();
            driverConnInfo.setServerInstanceId(split[2]);
            driverConnInfo.setSessionId(split[3]);
            Boolean exist = serviceCenter.driverServerExist(driverConnInfo.getServerInstanceId());
            if (exist) {
                SessionExchangeEvent sEvent = new SessionExchangeEvent();
                sEvent.setConnInfo(driverConnInfo);
                sEvent.setEvent(exchangeEvent);
                kafkaTemplate.send(driverConnInfo.getServerInstanceId(), JSON.toJSONString(sEvent));
            }
        }
        ack.acknowledge();
    }

    @KafkaListener(topics = {"#{serverConfig.getServiceName()}_subscribeExchangeEvent"}, groupId = "#{serverConfig.getServiceName()}")
    public void driverSubscribeExchangeEvent(ConsumerRecord<String, String> record, Acknowledgment ack) {
        String value = record.value();
        SubscribeExchangeEvent event = JSON.parseObject(value, SubscribeExchangeEvent.class);
        switch (event.getType()) {
            case "on":
//                开启监听
                log.info("DriverSubscribeExchangeEvent on: {}", event);
                List<String> keys = new ArrayList<>();
                keys.add("ExchangeDetail:" + event.getExchangeNo());
                keys.add("SessionsSubscribeExchange:" + event.getExchangeNo() + ":" + event.getConnInfo().getServerInstanceId() + ":" + event.getConnInfo().getSessionId());
//                采用Lua脚本确定原子性，1.设置监听实时消息 2.读取换电历史详情，预先推送
                List res = (List) redisTemplate.execute(subscribeScript, keys, event.getConnInfo());
                log.info("res:{}", res);
                for (Object s : res) {
                    ObjectMapper objectMapper = new ObjectMapper();
                    SessionExchangeEvent sEvent = new SessionExchangeEvent();
                    ExchangeEvent exchangeEvent = objectMapper.convertValue(s, ExchangeEvent.class);
                    sEvent.setEvent(exchangeEvent);
                    sEvent.setConnInfo(event.getConnInfo());
                    kafkaTemplate.send(event.getConnInfo().getServerInstanceId(), JSON.toJSONString(sEvent));
                }
                break;
            case "off":
//                关闭监听
                redisTemplate.delete("SessionsSubscribeExchange:" + event.getExchangeNo() + ":" + event.getConnInfo().getServerInstanceId() + ":" + event.getConnInfo().getSessionId());
                break;
            default:
                break;
        }
        ack.acknowledge();
    }

    @KafkaListener(topics = {"#{serverConfig.getServiceName()}_stationControlEvent"}, groupId = "#{serverConfig.getServiceName()}")
    public void driverControlStationEvent(ConsumerRecord<String, String> record, Acknowledgment ack) {
        String value = record.value();
        log.info("driverControlStationEvent:" + value);
        StationControlEvent event = JSON.parseObject(value, StationControlEvent.class);
        switch (event.getType()) {
            case "startExchange":
                String stationCode = event.getStationCode();
                Object o = redisTemplate.opsForValue().get("StationConn:" + stationCode);
                ObjectMapper objectMapper = new ObjectMapper();
                StationConnInfo stationConnInfo = objectMapper.convertValue(o, StationConnInfo.class);
                if (stationConnInfo == null) {
                    log.info("stationConnInfo({}) not exist", stationCode);
                    break;
                }
                event.setStationConnInfo(stationConnInfo);
                kafkaTemplate.send(stationConnInfo.getServerInstanceId(), JSON.toJSONString(event));
                break;
            default:
                break;
        }
        ack.acknowledge();
    }
}
